from typing import Literal, TypedDict, Dict, Any
import json

from .base_api_wrapper import RestAPIWrapper
from ..models_configurations.chatgpt_config import GPTConfiguration, chatGptModel

COMPLETIONS_API_URL = "https://api.openai.com/v1/chat/completions"


class GPTWrapperException(Exception):
    pass


class CompletionsRequestBody(TypedDict):
    model: chatGptModel
    messages: list[dict[Literal["role", "content"], str]]


class GPTWrapper(RestAPIWrapper):

    def __init__(self, configuration: GPTConfiguration) -> None:
        super().__init__(configuration)
        self.config = configuration
        self.api_url = COMPLETIONS_API_URL

    def _get_api_url(self) -> str:
        return self.api_url

    def _parse_response(self, response: Dict[str, Any]) -> Any:
        return {'response': response['choices'][0]['message']['content']}

    def _get_headers(self) -> dict[str, str]:
        return {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.config.api_key}"
        }

    def _get_body(self, prompt_message: str) -> CompletionsRequestBody:
        return json.dumps({
            "model": self.config.model,
            "messages": [
                {"role": "user",
                 "content": prompt_message}
            ]
        })



from .api_based_wrappers.gpt_wrapper import GPTConfiguration, GPTWrapper
from .api_based_wrappers.dalle_wrapper import DALLEWrapper, DALLEConfiguration
from .api_based_wrappers.hugging_face_wrapper import HuggingFaceConfiguration, HuggingFaceWrapper
import os
from typing import Dict, Any


def run_prompt_in_chatgpt_wrapper(prompt: str, config: GPTConfiguration | None = None) -> Dict[str, Any]:
    _config = config if config else GPTConfiguration(model='gpt-3.5-turbo',
                                                     api_key=os.getenv("OPENAI_API_KEY"))
    wrapper = GPTWrapper(configuration=_config)

    try:
        response = wrapper.prompt(prompt)
        print(response)
        return response
    except Exception as e:
        print(e)


def run_prompt_in_dalle_wrapper(prompt: str, config: DALLEConfiguration | None = None) -> Dict[str, Any]:
    _config = config if config else DALLEConfiguration(api_key=os.getenv("OPENAI_API_KEY"),
                                                       resolution='256x256',
                                                       num_of_images=4)
    wrapper = DALLEWrapper(configuration=_config)

    try:
        ret = wrapper.prompt(prompt)
        return ret
    except Exception as e:
        print(e)


def run_prompt_in_hugging_face_wrapper(prompt: str, config: HuggingFaceConfiguration) -> Dict[str, Any]:
    _config = config if config else HuggingFaceConfiguration(
        api_key=os.getenv("HUGGING_FACE_API_KEY"),
        model_id="distilbert-base-uncased")
    wrapper = HuggingFaceWrapper(configuration=_config)

    try:
        ret = wrapper.prompt(prompt)
        return ret
    except Exception as e:
        print(e)



from .base_config import BaseConfiguration
from .config_types import ConfigTypes

class HuggingFaceConfiguration(BaseConfiguration):
    def __init__(self, api_key: str, model_id: str) -> None:
        super().__init__(api_key)
        self.model_id = model_id

    def _get_config_type() -> ConfigTypes:
        return ConfigTypes.HUGGINGFACE